/*
 *   SPDX-FileCopyrightText: 2012 Aleix Pol Gonzalez <aleixpol@blue-systems.com>
 *
 *   SPDX-License-Identifier: LGPL-2.0-or-later
 */

#ifndef DISCOVERDECLARATIVEPLUGIN_H
#define DISCOVERDECLARATIVEPLUGIN_H

#include <QQmlExtensionPlugin>

class DiscoverDeclarativePlugin : public QQmlExtensionPlugin
{
    Q_OBJECT
    Q_PLUGIN_METADATA(IID "org.qt-project.Qt.QQmlExtensionInterface")
    public:
        void initializeEngine(QQmlEngine* engine, const char* uri) override;
        void registerTypes(const char* uri) override;
};

#endif
